@@ -1,28 +1,34 @@
 import errno
 import os
-from socket import socket as _original_socket
 import socket
 import sys
 import time
 import warnings
 
-from eventlet.support import get_errno, six
+import eventlet
 from eventlet.hubs import trampoline, notify_opened, IOClosed
+from eventlet.support import get_errno
 
 __all__ = [
     'GreenSocket', '_GLOBAL_DEFAULT_TIMEOUT', 'set_nonblocking',
-    'SOCKET_CLOSED', 'CONNECT_ERR', 'CONNECT_SUCCESS',
+    'SOCKET_BLOCKING', 'SOCKET_CLOSED', 'CONNECT_ERR', 'CONNECT_SUCCESS',
     'shutdown_safe', 'SSL',
+    'socket_timeout',
 ]
 
 BUFFER_SIZE = 4096
-CONNECT_ERR = set((errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK))
-CONNECT_SUCCESS = set((0, errno.EISCONN))
+CONNECT_ERR = {errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK}
+CONNECT_SUCCESS = {0, errno.EISCONN}
 if sys.platform[:3] == "win":
     CONNECT_ERR.add(errno.WSAEINVAL)   # Bug 67
 
-if six.PY2:
-    _python2_fileobject = socket._fileobject
+_original_socket = eventlet.patcher.original('socket').socket
+
+
+if sys.version_info >= (3, 10):
+    socket_timeout = socket.timeout  # Really, TimeoutError
+else:
+    socket_timeout = eventlet.timeout.wrap_is_timeout(socket.timeout)
 
 
 def socket_connect(descriptor, address):
@@ -34,14 +40,14 @@ def socket_connect(descriptor, address):
     if err in CONNECT_ERR:
         return None
     if err not in CONNECT_SUCCESS:
-        raise socket.error(err, errno.errorcode[err])
+        raise OSError(err, errno.errorcode[err])
     return descriptor
 
 
 def socket_checkerr(descriptor):
     err = descriptor.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
     if err not in CONNECT_SUCCESS:
-        raise socket.error(err, errno.errorcode[err])
+        raise OSError(err, errno.errorcode[err])
 
 
 def socket_accept(descriptor):
@@ -52,7 +58,7 @@ def socket_accept(descriptor):
     """
     try:
         return descriptor.accept()
-    except socket.error as e:
+    except OSError as e:
         if get_errno(e) == errno.EWOULDBLOCK:
             return None
         raise
@@ -60,13 +66,13 @@ def socket_accept(descriptor):
 
 if sys.platform[:3] == "win":
     # winsock sometimes throws ENOTCONN
-    SOCKET_BLOCKING = set((errno.EAGAIN, errno.EWOULDBLOCK,))
-    SOCKET_CLOSED = set((errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN))
+    SOCKET_BLOCKING = {errno.EAGAIN, errno.EWOULDBLOCK}
+    SOCKET_CLOSED = {errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN}
 else:
     # oddly, on linux/darwin, an unconnected socket is expected to block,
     # so we treat ENOTCONN the same as EWOULDBLOCK
-    SOCKET_BLOCKING = set((errno.EAGAIN, errno.EWOULDBLOCK, errno.ENOTCONN))
-    SOCKET_CLOSED = set((errno.ECONNRESET, errno.ESHUTDOWN, errno.EPIPE))
+    SOCKET_BLOCKING = {errno.EAGAIN, errno.EWOULDBLOCK, errno.ENOTCONN}
+    SOCKET_CLOSED = {errno.ECONNRESET, errno.ESHUTDOWN, errno.EPIPE}
 
 
 def set_nonblocking(fd):
@@ -110,7 +116,7 @@ except ImportError:
     _GLOBAL_DEFAULT_TIMEOUT = object()
 
 
-class GreenSocket(object):
+class GreenSocket:
     """
     Green version of socket.socket class, that is intended to be 100%
     API-compatible.
@@ -123,14 +129,14 @@ class GreenSocket(object):
     # This placeholder is to prevent __getattr__ from creating an infinite call loop
     fd = None
 
-    def __init__(self, family_or_realsock=socket.AF_INET, *args, **kwargs):
+    def __init__(self, family=socket.AF_INET, *args, **kwargs):
         should_set_nonblocking = kwargs.pop('set_nonblocking', True)
-        if isinstance(family_or_realsock, six.integer_types):
-            fd = _original_socket(family_or_realsock, *args, **kwargs)
+        if isinstance(family, int):
+            fd = _original_socket(family, *args, **kwargs)
             # Notify the hub that this is a newly-opened socket.
             notify_opened(fd.fileno())
         else:
-            fd = family_or_realsock
+            fd = family
 
         # import timeout from other socket, if it was there
         try:
@@ -138,7 +144,9 @@ class GreenSocket(object):
         except AttributeError:
             self._timeout = socket.getdefaulttimeout()
 
-        if should_set_nonblocking:
+        # Filter fd.fileno() != -1 so that won't call set non-blocking on
+        # closed socket
+        if should_set_nonblocking and fd.fileno() != -1:
             set_nonblocking(fd)
         self.fd = fd
         # when client calls setblocking(0) or settimeout(0) the socket must
@@ -164,14 +172,13 @@ class GreenSocket(object):
     def _sock(self):
         return self
 
-    if six.PY3:
-        def _get_io_refs(self):
-            return self.fd._io_refs
+    def _get_io_refs(self):
+        return self.fd._io_refs
 
-        def _set_io_refs(self, value):
-            self.fd._io_refs = value
+    def _set_io_refs(self, value):
+        self.fd._io_refs = value
 
-        _io_refs = property(_get_io_refs, _set_io_refs)
+    _io_refs = property(_get_io_refs, _set_io_refs)
 
     # Forward unknown attributes to fd, cache the value for future use.
     # I do not see any simple attribute which could be changed
@@ -206,16 +213,19 @@ class GreenSocket(object):
 
     def accept(self):
         if self.act_non_blocking:
-            return self.fd.accept()
+            res = self.fd.accept()
+            notify_opened(res[0].fileno())
+            return res
         fd = self.fd
+        _timeout_exc = socket_timeout('timed out')
         while True:
             res = socket_accept(fd)
             if res is not None:
                 client, addr = res
+                notify_opened(client.fileno())
                 set_nonblocking(client)
                 return type(self)(client), addr
-            self._trampoline(fd, read=True, timeout=self.gettimeout(),
-                             timeout_exc=socket.timeout("timed out"))
+            self._trampoline(fd, read=True, timeout=self.gettimeout(), timeout_exc=_timeout_exc)
 
     def _mark_as_closed(self):
         """ Mark this socket as being closed """
@@ -231,12 +241,13 @@ class GreenSocket(object):
         if self.act_non_blocking:
             return self.fd.connect(address)
         fd = self.fd
+        _timeout_exc = socket_timeout('timed out')
         if self.gettimeout() is None:
             while not socket_connect(fd, address):
                 try:
                     self._trampoline(fd, write=True)
                 except IOClosed:
-                    raise socket.error(errno.EBADFD)
+                    raise OSError(errno.EBADFD)
                 socket_checkerr(fd)
         else:
             end = time.time() + self.gettimeout()
@@ -244,13 +255,13 @@ class GreenSocket(object):
                 if socket_connect(fd, address):
                     return
                 if time.time() >= end:
-                    raise socket.timeout("timed out")
+                    raise _timeout_exc
+                timeout = end - time.time()
                 try:
-                    self._trampoline(fd, write=True, timeout=end - time.time(),
-                                     timeout_exc=socket.timeout("timed out"))
+                    self._trampoline(fd, write=True, timeout=timeout, timeout_exc=_timeout_exc)
                 except IOClosed:
                     # ... we need some workable errno here.
-                    raise socket.error(errno.EBADFD)
+                    raise OSError(errno.EBADFD)
                 socket_checkerr(fd)
 
     def connect_ex(self, address):
@@ -262,25 +273,28 @@ class GreenSocket(object):
                 try:
                     self._trampoline(fd, write=True)
                     socket_checkerr(fd)
-                except socket.error as ex:
+                except OSError as ex:
                     return get_errno(ex)
                 except IOClosed:
                     return errno.EBADFD
+                return 0
         else:
             end = time.time() + self.gettimeout()
+            timeout_exc = socket.timeout(errno.EAGAIN)
             while True:
                 try:
                     if socket_connect(fd, address):
                         return 0
                     if time.time() >= end:
-                        raise socket.timeout(errno.EAGAIN)
+                        raise timeout_exc
                     self._trampoline(fd, write=True, timeout=end - time.time(),
-                                     timeout_exc=socket.timeout(errno.EAGAIN))
+                                     timeout_exc=timeout_exc)
                     socket_checkerr(fd)
-                except socket.error as ex:
+                except OSError as ex:
                     return get_errno(ex)
                 except IOClosed:
                     return errno.EBADFD
+                return 0
 
     def dup(self, *args, **kw):
         sock = self.fd.dup(*args, **kw)
@@ -288,89 +302,88 @@ class GreenSocket(object):
         newsock.settimeout(self.gettimeout())
         return newsock
 
-    if six.PY3:
-        def makefile(self, *args, **kwargs):
-            return _original_socket.makefile(self, *args, **kwargs)
-    else:
-        def makefile(self, *args, **kwargs):
-            dupped = self.dup()
-            res = _python2_fileobject(dupped, *args, **kwargs)
-            if hasattr(dupped, "_drop"):
-                dupped._drop()
-            return res
+    def makefile(self, *args, **kwargs):
+        return _original_socket.makefile(self, *args, **kwargs)
 
     def makeGreenFile(self, *args, **kw):
         warnings.warn("makeGreenFile has been deprecated, please use "
                       "makefile instead", DeprecationWarning, stacklevel=2)
         return self.makefile(*args, **kw)
 
-    def recv(self, buflen, flags=0):
-        fd = self.fd
+    def _read_trampoline(self):
+        self._trampoline(
+            self.fd,
+            read=True,
+            timeout=self.gettimeout(),
+            timeout_exc=socket_timeout('timed out'))
+
+    def _recv_loop(self, recv_meth, empty_val, *args):
         if self.act_non_blocking:
-            return fd.recv(buflen, flags)
+            return recv_meth(*args)
+
         while True:
             try:
-                return fd.recv(buflen, flags)
-            except socket.error as e:
+                # recv: bufsize=0?
+                # recv_into: buffer is empty?
+                # This is needed because behind the scenes we use sockets in
+                # nonblocking mode and builtin recv* methods. Attempting to read
+                # 0 bytes from a nonblocking socket using a builtin recv* method
+                # does not raise a timeout exception. Since we're simulating
+                # a blocking socket here we need to produce a timeout exception
+                # if needed, hence the call to trampoline.
+                if not args[0]:
+                    self._read_trampoline()
+                return recv_meth(*args)
+            except OSError as e:
                 if get_errno(e) in SOCKET_BLOCKING:
                     pass
                 elif get_errno(e) in SOCKET_CLOSED:
-                    return ''
+                    return empty_val
                 else:
                     raise
+
             try:
-                self._trampoline(
-                    fd,
-                    read=True,
-                    timeout=self.gettimeout(),
-                    timeout_exc=socket.timeout("timed out"))
+                self._read_trampoline()
             except IOClosed as e:
                 # Perhaps we should return '' instead?
                 raise EOFError()
 
-    def recvfrom(self, *args):
-        if not self.act_non_blocking:
-            self._trampoline(self.fd, read=True, timeout=self.gettimeout(),
-                             timeout_exc=socket.timeout("timed out"))
-        return self.fd.recvfrom(*args)
+    def recv(self, bufsize, flags=0):
+        return self._recv_loop(self.fd.recv, b'', bufsize, flags)
 
-    def recvfrom_into(self, *args):
-        if not self.act_non_blocking:
-            self._trampoline(self.fd, read=True, timeout=self.gettimeout(),
-                             timeout_exc=socket.timeout("timed out"))
-        return self.fd.recvfrom_into(*args)
+    def recvfrom(self, bufsize, flags=0):
+        return self._recv_loop(self.fd.recvfrom, b'', bufsize, flags)
 
-    def recv_into(self, *args):
-        if not self.act_non_blocking:
-            self._trampoline(self.fd, read=True, timeout=self.gettimeout(),
-                             timeout_exc=socket.timeout("timed out"))
-        return self.fd.recv_into(*args)
+    def recv_into(self, buffer, nbytes=0, flags=0):
+        return self._recv_loop(self.fd.recv_into, 0, buffer, nbytes, flags)
 
-    def send(self, data, flags=0):
-        fd = self.fd
+    def recvfrom_into(self, buffer, nbytes=0, flags=0):
+        return self._recv_loop(self.fd.recvfrom_into, 0, buffer, nbytes, flags)
+
+    def _send_loop(self, send_method, data, *args):
         if self.act_non_blocking:
-            return fd.send(data, flags)
+            return send_method(data, *args)
 
-        # blocking socket behavior - sends all, blocks if the buffer is full
-        total_sent = 0
-        len_data = len(data)
-        while 1:
+        _timeout_exc = socket_timeout('timed out')
+        while True:
             try:
-                total_sent += fd.send(data[total_sent:], flags)
-            except socket.error as e:
-                if get_errno(e) not in SOCKET_BLOCKING:
+                return send_method(data, *args)
+            except OSError as e:
+                eno = get_errno(e)
+                if eno == errno.ENOTCONN or eno not in SOCKET_BLOCKING:
                     raise
 
-            if total_sent == len_data:
-                break
-
             try:
                 self._trampoline(self.fd, write=True, timeout=self.gettimeout(),
-                                 timeout_exc=socket.timeout("timed out"))
+                                 timeout_exc=_timeout_exc)
             except IOClosed:
-                raise socket.error(errno.ECONNRESET, 'Connection closed by another thread')
+                raise OSError(errno.ECONNRESET, 'Connection closed by another thread')
 
-        return total_sent
+    def send(self, data, flags=0):
+        return self._send_loop(self.fd.send, data, flags)
+
+    def sendto(self, data, *args):
+        return self._send_loop(self.fd.sendto, data, *args)
 
     def sendall(self, data, flags=0):
         tail = self.send(data, flags)
@@ -378,10 +391,6 @@ class GreenSocket(object):
         while tail < len_data:
             tail += self.send(data[tail:], flags)
 
-    def sendto(self, *args):
-        self._trampoline(self.fd, write=True)
-        return self.fd.sendto(*args)
-
     def setblocking(self, flag):
         if flag:
             self.act_non_blocking = False
@@ -411,12 +420,11 @@ class GreenSocket(object):
     def gettimeout(self):
         return self._timeout
 
-    if "__pypy__" in sys.builtin_module_names:
-        def _reuse(self):
-            getattr(self.fd, '_sock', self.fd)._reuse()
+    def __enter__(self):
+        return self
 
-        def _drop(self):
-            getattr(self.fd, '_sock', self.fd)._drop()
+    def __exit__(self, *args):
+        self.close()
 
 
 def _operation_on_closed_file(*args, **kwargs):
@@ -426,7 +434,7 @@ def _operation_on_closed_file(*args, **kwargs):
 greenpipe_doc = """
     GreenPipe is a cooperative replacement for file class.
     It will cooperate on pipes. It will block on regular file.
-    Differneces from file class:
+    Differences from file class:
     - mode is r/w property. Should re r/o
     - encoding property not implemented
     - write/writelines will not raise TypeError exception when non-string data is written
@@ -440,7 +448,7 @@ try:
     from OpenSSL import SSL
 except ImportError:
     # pyOpenSSL not installed, define exceptions anyway for convenience
-    class SSL(object):
+    class SSL:
         class WantWriteError(Exception):
             pass
 
@@ -455,11 +463,11 @@ except ImportError:
 
 
 def shutdown_safe(sock):
-    """ Shuts down the socket. This is a convenience method for
+    """Shuts down the socket. This is a convenience method for
     code that wants to gracefully handle regular sockets, SSL.Connection
-    sockets from PyOpenSSL and ssl.SSLSocket objects from Python 2.6
-    interchangeably.  Both types of ssl socket require a shutdown() before
-    close, but they have different arity on their shutdown method.
+    sockets from PyOpenSSL and ssl.SSLSocket objects from Python 2.7 interchangeably.
+    Both types of ssl socket require a shutdown() before close,
+    but they have different arity on their shutdown method.
 
     Regular sockets don't need a shutdown before close, but it doesn't hurt.
     """
@@ -470,8 +478,8 @@ def shutdown_safe(sock):
         except TypeError:
             # SSL.Connection
             return sock.shutdown()
-    except socket.error as e:
+    except OSError as e:
         # we don't care if the socket is already closed;
         # this will often be the case in an http server context
-        if get_errno(e) not in (errno.ENOTCONN, errno.EBADF):
+        if get_errno(e) not in (errno.ENOTCONN, errno.EBADF, errno.ENOTSOCK):
             raise
