@@ -1,20 +1,28 @@
-'''Test cases for db_pool
-'''
-from __future__ import print_function
-
-import sys
 import os
+import sys
 import traceback
-from unittest import TestCase, main
 
-from tests import mock, skip_unless, skip_with_pyevent, get_database_auth
-from eventlet import event
 from eventlet import db_pool
-from eventlet.support import six
 import eventlet
+import eventlet.tpool
+import tests
+import tests.mock
+
+psycopg2 = None
+try:
+    import psycopg2
+    import psycopg2.extensions
+except ImportError:
+    pass
+
+MySQLdb = None
+try:
+    import MySQLdb
+except ImportError:
+    pass
 
 
-class DBTester(object):
+class DBTester:
     __test__ = False  # so that nose doesn't try to execute this directly
 
     def setUp(self):
@@ -53,7 +61,7 @@ class DBTester(object):
 
 
 # silly mock class
-class Mock(object):
+class Mock:
     pass
 
 
@@ -61,7 +69,7 @@ class DBConnectionPool(DBTester):
     __test__ = False  # so that nose doesn't try to execute this directly
 
     def setUp(self):
-        super(DBConnectionPool, self).setUp()
+        super().setUp()
         self.pool = self.create_pool()
         self.connection = self.pool.get()
 
@@ -69,7 +77,7 @@ class DBConnectionPool(DBTester):
         if self.connection:
             self.pool.put(self.connection)
         self.pool.clear()
-        super(DBConnectionPool, self).tearDown()
+        super().tearDown()
 
     def assert_cursor_works(self, cursor):
         cursor.execute("select 1")
@@ -129,7 +137,7 @@ class DBConnectionPool(DBTester):
 
     def fill_up_table(self, conn):
         curs = conn.cursor()
-        for i in six.moves.range(1000):
+        for i in range(1000):
             curs.execute('insert into test_table (value_int) values (%s)' % i)
         conn.commit()
 
@@ -141,7 +149,7 @@ class DBConnectionPool(DBTester):
         curs = conn.cursor()
         results = []
         SHORT_QUERY = "select * from test_table"
-        evt = event.Event()
+        evt = eventlet.Event()
 
         def a_query():
             self.assert_cursor_works(curs)
@@ -218,8 +226,8 @@ class DBConnectionPool(DBTester):
         self.assertEqual(len(self.pool.free_items), 0)
 
     def test_unwrap_connection(self):
-        self.assert_(isinstance(self.connection,
-                                db_pool.GenericConnectionWrapper))
+        self.assertTrue(isinstance(self.connection,
+                                   db_pool.GenericConnectionWrapper))
         conn = self.pool._unwrap_connection(self.connection)
         assert not isinstance(conn, db_pool.GenericConnectionWrapper)
 
@@ -282,7 +290,7 @@ class DBConnectionPool(DBTester):
         self.connection = self.pool.get()
         self.assertEqual(self.pool.free(), 0)
         self.assertEqual(self.pool.waiting(), 0)
-        e = event.Event()
+        e = eventlet.Event()
 
         def retrieve(pool, ev):
             c = pool.get()
@@ -308,17 +316,17 @@ class DBConnectionPool(DBTester):
         self.assertEqual(self.pool.free(), 1)
 
 
-class DummyConnection(object):
+class DummyConnection:
     def rollback(self):
         pass
 
 
-class DummyDBModule(object):
+class DummyDBModule:
     def connect(self, *args, **kwargs):
         return DummyConnection()
 
 
-class RaisingDBModule(object):
+class RaisingDBModule:
     def connect(self, *args, **kw):
         raise RuntimeError()
 
@@ -337,14 +345,12 @@ class TpoolConnectionPool(DBConnectionPool):
             connect_timeout=connect_timeout,
             **self._auth)
 
-    @skip_with_pyevent
     def setUp(self):
-        super(TpoolConnectionPool, self).setUp()
+        super().setUp()
 
     def tearDown(self):
-        super(TpoolConnectionPool, self).tearDown()
-        from eventlet import tpool
-        tpool.killall()
+        super().tearDown()
+        eventlet.tpool.killall()
 
 
 class RawConnectionPool(DBConnectionPool):
@@ -373,7 +379,7 @@ def test_raw_pool_issue_125():
 
 
 def test_raw_pool_custom_cleanup_ok():
-    cleanup_mock = mock.Mock()
+    cleanup_mock = tests.mock.Mock()
     pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup_mock)
     conn = pool.get()
     pool.put(conn)
@@ -385,7 +391,7 @@ def test_raw_pool_custom_cleanup_ok():
 
 
 def test_raw_pool_custom_cleanup_arg_error():
-    cleanup_mock = mock.Mock(side_effect=NotImplementedError)
+    cleanup_mock = tests.mock.Mock(side_effect=NotImplementedError)
     pool = db_pool.RawConnectionPool(DummyDBModule())
     conn = pool.get()
     pool.put(conn, cleanup=cleanup_mock)
@@ -427,29 +433,25 @@ def test_raw_pool_clear_update_current_size():
     assert len(pool.free_items) == 0
 
 
-get_auth = get_database_auth
-
-
 def mysql_requirement(_f):
     verbose = os.environ.get('eventlet_test_mysql_verbose')
-    try:
-        import MySQLdb
-        try:
-            auth = get_auth()['MySQLdb'].copy()
-            MySQLdb.connect(**auth)
-            return True
-        except MySQLdb.OperationalError:
-            if verbose:
-                print(">> Skipping mysql tests, error when connecting:", file=sys.stderr)
-                traceback.print_exc()
-            return False
-    except ImportError:
+    if MySQLdb is None:
         if verbose:
             print(">> Skipping mysql tests, MySQLdb not importable", file=sys.stderr)
         return False
 
+    try:
+        auth = tests.get_database_auth()['MySQLdb'].copy()
+        MySQLdb.connect(**auth)
+        return True
+    except MySQLdb.OperationalError:
+        if verbose:
+            print(">> Skipping mysql tests, error when connecting:", file=sys.stderr)
+            traceback.print_exc()
+        return False
 
-class MysqlConnectionPool(object):
+
+class MysqlConnectionPool:
     dummy_table_sql = """CREATE TEMPORARY TABLE test_table
         (
         row_id INTEGER PRIMARY KEY AUTO_INCREMENT,
@@ -463,15 +465,14 @@ class MysqlConnectionPool(object):
         created TIMESTAMP
         ) ENGINE=InnoDB;"""
 
-    @skip_unless(mysql_requirement)
+    @tests.skip_unless(mysql_requirement)
     def setUp(self):
-        import MySQLdb
         self._dbmodule = MySQLdb
-        self._auth = get_auth()['MySQLdb']
-        super(MysqlConnectionPool, self).setUp()
+        self._auth = tests.get_database_auth()['MySQLdb']
+        super().setUp()
 
     def tearDown(self):
-        super(MysqlConnectionPool, self).tearDown()
+        super().tearDown()
 
     def create_db(self):
         auth = self._auth.copy()
@@ -488,35 +489,34 @@ class MysqlConnectionPool(object):
 
     def drop_db(self):
         db = self._dbmodule.connect(**self._auth).cursor()
-        db.execute("drop database " + self._auth['db'])
+        db.execute("drop database IF EXISTS " + self._auth['db'])
         db.close()
         del db
 
 
-class Test01MysqlTpool(MysqlConnectionPool, TpoolConnectionPool, TestCase):
+class Test01MysqlTpool(MysqlConnectionPool, TpoolConnectionPool, tests.LimitedTestCase):
     __test__ = True
 
 
-class Test02MysqlRaw(MysqlConnectionPool, RawConnectionPool, TestCase):
+class Test02MysqlRaw(MysqlConnectionPool, RawConnectionPool, tests.LimitedTestCase):
     __test__ = True
 
 
 def postgres_requirement(_f):
-    try:
-        import psycopg2
-        try:
-            auth = get_auth()['psycopg2'].copy()
-            psycopg2.connect(**auth)
-            return True
-        except psycopg2.OperationalError:
-            print("Skipping postgres tests, error when connecting")
-            return False
-    except ImportError:
+    if psycopg2 is None:
         print("Skipping postgres tests, psycopg2 not importable")
         return False
 
+    try:
+        auth = tests.get_database_auth()['psycopg2'].copy()
+        psycopg2.connect(**auth)
+        return True
+    except psycopg2.OperationalError:
+        print("Skipping postgres tests, error when connecting")
+        return False
 
-class Psycopg2ConnectionPool(object):
+
+class Psycopg2ConnectionPool:
     dummy_table_sql = """CREATE TEMPORARY TABLE test_table
         (
         row_id SERIAL PRIMARY KEY,
@@ -529,15 +529,14 @@ class Psycopg2ConnectionPool(object):
         created TIMESTAMP
         );"""
 
-    @skip_unless(postgres_requirement)
+    @tests.skip_unless(postgres_requirement)
     def setUp(self):
-        import psycopg2
         self._dbmodule = psycopg2
-        self._auth = get_auth()['psycopg2']
-        super(Psycopg2ConnectionPool, self).setUp()
+        self._auth = tests.get_database_auth()['psycopg2']
+        super().setUp()
 
     def tearDown(self):
-        super(Psycopg2ConnectionPool, self).tearDown()
+        super().tearDown()
 
     def create_db(self):
         dbname = 'test%s' % os.getpid()
@@ -561,12 +560,12 @@ class Psycopg2ConnectionPool(object):
         conn = self._dbmodule.connect(**auth)
         conn.set_isolation_level(0)
         db = conn.cursor()
-        db.execute("drop database " + self._auth['database'])
+        db.execute("drop database IF EXISTS " + self._auth['database'])
         db.close()
         conn.close()
 
 
-class TestPsycopg2Base(TestCase):
+class TestPsycopg2Base(tests.LimitedTestCase):
     __test__ = False
 
     def test_cursor_works_as_context_manager(self):
@@ -575,6 +574,9 @@ class TestPsycopg2Base(TestCase):
             row = c.fetchone()
             assert row == (1,)
 
+    def test_set_isolation_level(self):
+        self.connection.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
+
 
 class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestPsycopg2Base):
     __test__ = True
@@ -582,7 +584,3 @@ class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestPsyco
 
 class Test02Psycopg2Raw(Psycopg2ConnectionPool, RawConnectionPool, TestPsycopg2Base):
     __test__ = True
-
-
-if __name__ == '__main__':
-    main()
