@@ -514,14 +514,33 @@ def test_threadpoolexecutor():
     tests.run_isolated('patcher_threadpoolexecutor.py')
 
 
+FORK_REASON = "fork() doesn't work well on macOS, and definitely doesn't work on Windows"
+
+
 @pytest.mark.skipif(
     not sys.platform.startswith("linux"),
-    reason="fork() doesn't work well on macOS, and definitely doesn't work on Windows"
+    reason=FORK_REASON
 )
 def test_fork_after_monkey_patch():
     tests.run_isolated('patcher_fork_after_monkey_patch.py')
 
 
+@pytest.mark.skipif(
+    not sys.platform.startswith("linux"),
+    reason=FORK_REASON
+)
+def test_fork_after_monkey_patch_threading():
+    tests.run_isolated('fork_in_main_thread.py')
+
+
+@pytest.mark.skipif(
+    not sys.platform.startswith("linux"),
+    reason=FORK_REASON
+)
+def test_fork_in_thread_after_monkey_patch_threading():
+    tests.run_isolated('fork_in_thread.py')
+
+
 def test_builtin():
     tests.run_isolated('patcher_builtin.py')
 
